from DappleySdk.lib.protobuf import transactionBase_pb2


class TxOutput(object):
    def __init__(self, value, public_key_hash, contract):
        self.value = value
        self.public_key_hash = public_key_hash
        self.contract = contract

    def to_proto(self):
        proto_tx_out = transactionBase_pb2.TXOutput(value=self.value, contract=self.contract,
                                                    public_key_hash=self.public_key_hash)
        return proto_tx_out

    def parse_proto(self, proto_tx_out):
        self.value = proto_tx_out.value
        self.public_key_hash = proto_tx_out.public_key_hash
        self.contract = proto_tx_out.contract



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import rpc_pb2 as rpc__pb2


class RpcServiceStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RpcGetVersion = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetVersion',
                request_serializer=rpc__pb2.GetVersionRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetVersionResponse.FromString,
                )
        self.RpcGetBalance = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetBalance',
                request_serializer=rpc__pb2.GetBalanceRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetBalanceResponse.FromString,
                )
        self.RpcGetBlockchainInfo = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetBlockchainInfo',
                request_serializer=rpc__pb2.GetBlockchainInfoRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetBlockchainInfoResponse.FromString,
                )
        self.RpcGetUTXO = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetUTXO',
                request_serializer=rpc__pb2.GetUTXORequest.SerializeToString,
                response_deserializer=rpc__pb2.GetUTXOResponse.FromString,
                )
        self.RpcGetBlocks = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetBlocks',
                request_serializer=rpc__pb2.GetBlocksRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetBlocksResponse.FromString,
                )
        self.RpcGetBlockByHash = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetBlockByHash',
                request_serializer=rpc__pb2.GetBlockByHashRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetBlockByHashResponse.FromString,
                )
        self.RpcGetBlockByHeight = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetBlockByHeight',
                request_serializer=rpc__pb2.GetBlockByHeightRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetBlockByHeightResponse.FromString,
                )
        self.RpcSendTransaction = channel.unary_unary(
                '/rpcpb.RpcService/RpcSendTransaction',
                request_serializer=rpc__pb2.SendTransactionRequest.SerializeToString,
                response_deserializer=rpc__pb2.SendTransactionResponse.FromString,
                )
        self.RpcSendBatchTransaction = channel.unary_unary(
                '/rpcpb.RpcService/RpcSendBatchTransaction',
                request_serializer=rpc__pb2.SendBatchTransactionRequest.SerializeToString,
                response_deserializer=rpc__pb2.SendBatchTransactionResponse.FromString,
                )
        self.RpcGetNewTransaction = channel.unary_stream(
                '/rpcpb.RpcService/RpcGetNewTransaction',
                request_serializer=rpc__pb2.GetNewTransactionRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetNewTransactionResponse.FromString,
                )
        self.RpcSubscribe = channel.unary_stream(
                '/rpcpb.RpcService/RpcSubscribe',
                request_serializer=rpc__pb2.SubscribeRequest.SerializeToString,
                response_deserializer=rpc__pb2.SubscribeResponse.FromString,
                )
        self.RpcGetAllTransactionsFromTxPool = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetAllTransactionsFromTxPool',
                request_serializer=rpc__pb2.GetAllTransactionsRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetAllTransactionsResponse.FromString,
                )
        self.RpcGetLastIrreversibleBlock = channel.unary_unary(
                '/rpcpb.RpcService/RpcGetLastIrreversibleBlock',
                request_serializer=rpc__pb2.GetLastIrreversibleBlockRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetLastIrreversibleBlockResponse.FromString,
                )
        self.RpcEstimateGas = channel.unary_unary(
                '/rpcpb.RpcService/RpcEstimateGas',
                request_serializer=rpc__pb2.EstimateGasRequest.SerializeToString,
                response_deserializer=rpc__pb2.EstimateGasResponse.FromString,
                )
        self.RpcGasPrice = channel.unary_unary(
                '/rpcpb.RpcService/RpcGasPrice',
                request_serializer=rpc__pb2.GasPriceRequest.SerializeToString,
                response_deserializer=rpc__pb2.GasPriceResponse.FromString,
                )
        self.RpcContractQuery = channel.unary_unary(
                '/rpcpb.RpcService/RpcContractQuery',
                request_serializer=rpc__pb2.ContractQueryRequest.SerializeToString,
                response_deserializer=rpc__pb2.ContractQueryResponse.FromString,
                )


class RpcServiceServicer(object):
    """Missing associated documentation comment in .proto file"""

    def RpcGetVersion(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetBalance(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetBlockchainInfo(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetUTXO(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetBlocks(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetBlockByHash(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetBlockByHeight(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSendTransaction(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSendBatchTransaction(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetNewTransaction(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSubscribe(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetAllTransactionsFromTxPool(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetLastIrreversibleBlock(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcEstimateGas(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGasPrice(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcContractQuery(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RpcServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RpcGetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetVersion,
                    request_deserializer=rpc__pb2.GetVersionRequest.FromString,
                    response_serializer=rpc__pb2.GetVersionResponse.SerializeToString,
            ),
            'RpcGetBalance': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetBalance,
                    request_deserializer=rpc__pb2.GetBalanceRequest.FromString,
                    response_serializer=rpc__pb2.GetBalanceResponse.SerializeToString,
            ),
            'RpcGetBlockchainInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetBlockchainInfo,
                    request_deserializer=rpc__pb2.GetBlockchainInfoRequest.FromString,
                    response_serializer=rpc__pb2.GetBlockchainInfoResponse.SerializeToString,
            ),
            'RpcGetUTXO': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetUTXO,
                    request_deserializer=rpc__pb2.GetUTXORequest.FromString,
                    response_serializer=rpc__pb2.GetUTXOResponse.SerializeToString,
            ),
            'RpcGetBlocks': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetBlocks,
                    request_deserializer=rpc__pb2.GetBlocksRequest.FromString,
                    response_serializer=rpc__pb2.GetBlocksResponse.SerializeToString,
            ),
            'RpcGetBlockByHash': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetBlockByHash,
                    request_deserializer=rpc__pb2.GetBlockByHashRequest.FromString,
                    response_serializer=rpc__pb2.GetBlockByHashResponse.SerializeToString,
            ),
            'RpcGetBlockByHeight': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetBlockByHeight,
                    request_deserializer=rpc__pb2.GetBlockByHeightRequest.FromString,
                    response_serializer=rpc__pb2.GetBlockByHeightResponse.SerializeToString,
            ),
            'RpcSendTransaction': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcSendTransaction,
                    request_deserializer=rpc__pb2.SendTransactionRequest.FromString,
                    response_serializer=rpc__pb2.SendTransactionResponse.SerializeToString,
            ),
            'RpcSendBatchTransaction': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcSendBatchTransaction,
                    request_deserializer=rpc__pb2.SendBatchTransactionRequest.FromString,
                    response_serializer=rpc__pb2.SendBatchTransactionResponse.SerializeToString,
            ),
            'RpcGetNewTransaction': grpc.unary_stream_rpc_method_handler(
                    servicer.RpcGetNewTransaction,
                    request_deserializer=rpc__pb2.GetNewTransactionRequest.FromString,
                    response_serializer=rpc__pb2.GetNewTransactionResponse.SerializeToString,
            ),
            'RpcSubscribe': grpc.unary_stream_rpc_method_handler(
                    servicer.RpcSubscribe,
                    request_deserializer=rpc__pb2.SubscribeRequest.FromString,
                    response_serializer=rpc__pb2.SubscribeResponse.SerializeToString,
            ),
            'RpcGetAllTransactionsFromTxPool': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetAllTransactionsFromTxPool,
                    request_deserializer=rpc__pb2.GetAllTransactionsRequest.FromString,
                    response_serializer=rpc__pb2.GetAllTransactionsResponse.SerializeToString,
            ),
            'RpcGetLastIrreversibleBlock': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetLastIrreversibleBlock,
                    request_deserializer=rpc__pb2.GetLastIrreversibleBlockRequest.FromString,
                    response_serializer=rpc__pb2.GetLastIrreversibleBlockResponse.SerializeToString,
            ),
            'RpcEstimateGas': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcEstimateGas,
                    request_deserializer=rpc__pb2.EstimateGasRequest.FromString,
                    response_serializer=rpc__pb2.EstimateGasResponse.SerializeToString,
            ),
            'RpcGasPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGasPrice,
                    request_deserializer=rpc__pb2.GasPriceRequest.FromString,
                    response_serializer=rpc__pb2.GasPriceResponse.SerializeToString,
            ),
            'RpcContractQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcContractQuery,
                    request_deserializer=rpc__pb2.ContractQueryRequest.FromString,
                    response_serializer=rpc__pb2.ContractQueryResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rpcpb.RpcService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RpcService(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def RpcGetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetVersion',
            rpc__pb2.GetVersionRequest.SerializeToString,
            rpc__pb2.GetVersionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetBalance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetBalance',
            rpc__pb2.GetBalanceRequest.SerializeToString,
            rpc__pb2.GetBalanceResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetBlockchainInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetBlockchainInfo',
            rpc__pb2.GetBlockchainInfoRequest.SerializeToString,
            rpc__pb2.GetBlockchainInfoResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetUTXO(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetUTXO',
            rpc__pb2.GetUTXORequest.SerializeToString,
            rpc__pb2.GetUTXOResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetBlocks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetBlocks',
            rpc__pb2.GetBlocksRequest.SerializeToString,
            rpc__pb2.GetBlocksResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetBlockByHash(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetBlockByHash',
            rpc__pb2.GetBlockByHashRequest.SerializeToString,
            rpc__pb2.GetBlockByHashResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetBlockByHeight(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetBlockByHeight',
            rpc__pb2.GetBlockByHeightRequest.SerializeToString,
            rpc__pb2.GetBlockByHeightResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSendTransaction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcSendTransaction',
            rpc__pb2.SendTransactionRequest.SerializeToString,
            rpc__pb2.SendTransactionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSendBatchTransaction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcSendBatchTransaction',
            rpc__pb2.SendBatchTransactionRequest.SerializeToString,
            rpc__pb2.SendBatchTransactionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetNewTransaction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rpcpb.RpcService/RpcGetNewTransaction',
            rpc__pb2.GetNewTransactionRequest.SerializeToString,
            rpc__pb2.GetNewTransactionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rpcpb.RpcService/RpcSubscribe',
            rpc__pb2.SubscribeRequest.SerializeToString,
            rpc__pb2.SubscribeResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetAllTransactionsFromTxPool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetAllTransactionsFromTxPool',
            rpc__pb2.GetAllTransactionsRequest.SerializeToString,
            rpc__pb2.GetAllTransactionsResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetLastIrreversibleBlock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGetLastIrreversibleBlock',
            rpc__pb2.GetLastIrreversibleBlockRequest.SerializeToString,
            rpc__pb2.GetLastIrreversibleBlockResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcEstimateGas(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcEstimateGas',
            rpc__pb2.EstimateGasRequest.SerializeToString,
            rpc__pb2.EstimateGasResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGasPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcGasPrice',
            rpc__pb2.GasPriceRequest.SerializeToString,
            rpc__pb2.GasPriceResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcContractQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.RpcService/RpcContractQuery',
            rpc__pb2.ContractQueryRequest.SerializeToString,
            rpc__pb2.ContractQueryResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)


class AdminServiceStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RpcAddPeer = channel.unary_unary(
                '/rpcpb.AdminService/RpcAddPeer',
                request_serializer=rpc__pb2.AddPeerRequest.SerializeToString,
                response_deserializer=rpc__pb2.AddPeerResponse.FromString,
                )
        self.RpcSend = channel.unary_unary(
                '/rpcpb.AdminService/RpcSend',
                request_serializer=rpc__pb2.SendRequest.SerializeToString,
                response_deserializer=rpc__pb2.SendResponse.FromString,
                )
        self.RpcGetPeerInfo = channel.unary_unary(
                '/rpcpb.AdminService/RpcGetPeerInfo',
                request_serializer=rpc__pb2.GetPeerInfoRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetPeerInfoResponse.FromString,
                )
        self.RpcSendFromMiner = channel.unary_unary(
                '/rpcpb.AdminService/RpcSendFromMiner',
                request_serializer=rpc__pb2.SendFromMinerRequest.SerializeToString,
                response_deserializer=rpc__pb2.SendFromMinerResponse.FromString,
                )
        self.RpcAddProducer = channel.unary_unary(
                '/rpcpb.AdminService/RpcAddProducer',
                request_serializer=rpc__pb2.AddProducerRequest.SerializeToString,
                response_deserializer=rpc__pb2.AddProducerResponse.FromString,
                )
        self.RpcUnlockAccount = channel.unary_unary(
                '/rpcpb.AdminService/RpcUnlockAccount',
                request_serializer=rpc__pb2.UnlockAccountRequest.SerializeToString,
                response_deserializer=rpc__pb2.UnlockAccountResponse.FromString,
                )


class AdminServiceServicer(object):
    """Missing associated documentation comment in .proto file"""

    def RpcAddPeer(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSend(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetPeerInfo(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSendFromMiner(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcAddProducer(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcUnlockAccount(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AdminServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RpcAddPeer': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcAddPeer,
                    request_deserializer=rpc__pb2.AddPeerRequest.FromString,
                    response_serializer=rpc__pb2.AddPeerResponse.SerializeToString,
            ),
            'RpcSend': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcSend,
                    request_deserializer=rpc__pb2.SendRequest.FromString,
                    response_serializer=rpc__pb2.SendResponse.SerializeToString,
            ),
            'RpcGetPeerInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetPeerInfo,
                    request_deserializer=rpc__pb2.GetPeerInfoRequest.FromString,
                    response_serializer=rpc__pb2.GetPeerInfoResponse.SerializeToString,
            ),
            'RpcSendFromMiner': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcSendFromMiner,
                    request_deserializer=rpc__pb2.SendFromMinerRequest.FromString,
                    response_serializer=rpc__pb2.SendFromMinerResponse.SerializeToString,
            ),
            'RpcAddProducer': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcAddProducer,
                    request_deserializer=rpc__pb2.AddProducerRequest.FromString,
                    response_serializer=rpc__pb2.AddProducerResponse.SerializeToString,
            ),
            'RpcUnlockAccount': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcUnlockAccount,
                    request_deserializer=rpc__pb2.UnlockAccountRequest.FromString,
                    response_serializer=rpc__pb2.UnlockAccountResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rpcpb.AdminService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AdminService(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def RpcAddPeer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcAddPeer',
            rpc__pb2.AddPeerRequest.SerializeToString,
            rpc__pb2.AddPeerResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSend(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcSend',
            rpc__pb2.SendRequest.SerializeToString,
            rpc__pb2.SendResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetPeerInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcGetPeerInfo',
            rpc__pb2.GetPeerInfoRequest.SerializeToString,
            rpc__pb2.GetPeerInfoResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSendFromMiner(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcSendFromMiner',
            rpc__pb2.SendFromMinerRequest.SerializeToString,
            rpc__pb2.SendFromMinerResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcAddProducer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcAddProducer',
            rpc__pb2.AddProducerRequest.SerializeToString,
            rpc__pb2.AddProducerResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcUnlockAccount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.AdminService/RpcUnlockAccount',
            rpc__pb2.UnlockAccountRequest.SerializeToString,
            rpc__pb2.UnlockAccountResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)


class MetricServiceStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RpcGetStats = channel.unary_unary(
                '/rpcpb.MetricService/RpcGetStats',
                request_serializer=rpc__pb2.MetricsServiceRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetStatsResponse.FromString,
                )
        self.RpcGetNodeConfig = channel.unary_unary(
                '/rpcpb.MetricService/RpcGetNodeConfig',
                request_serializer=rpc__pb2.MetricsServiceRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetNodeConfigResponse.FromString,
                )
        self.RpcSetNodeConfig = channel.unary_unary(
                '/rpcpb.MetricService/RpcSetNodeConfig',
                request_serializer=rpc__pb2.SetNodeConfigRequest.SerializeToString,
                response_deserializer=rpc__pb2.GetNodeConfigResponse.FromString,
                )


class MetricServiceServicer(object):
    """Missing associated documentation comment in .proto file"""

    def RpcGetStats(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcGetNodeConfig(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RpcSetNodeConfig(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MetricServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RpcGetStats': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetStats,
                    request_deserializer=rpc__pb2.MetricsServiceRequest.FromString,
                    response_serializer=rpc__pb2.GetStatsResponse.SerializeToString,
            ),
            'RpcGetNodeConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcGetNodeConfig,
                    request_deserializer=rpc__pb2.MetricsServiceRequest.FromString,
                    response_serializer=rpc__pb2.GetNodeConfigResponse.SerializeToString,
            ),
            'RpcSetNodeConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.RpcSetNodeConfig,
                    request_deserializer=rpc__pb2.SetNodeConfigRequest.FromString,
                    response_serializer=rpc__pb2.GetNodeConfigResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rpcpb.MetricService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MetricService(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def RpcGetStats(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.MetricService/RpcGetStats',
            rpc__pb2.MetricsServiceRequest.SerializeToString,
            rpc__pb2.GetStatsResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcGetNodeConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.MetricService/RpcGetNodeConfig',
            rpc__pb2.MetricsServiceRequest.SerializeToString,
            rpc__pb2.GetNodeConfigResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RpcSetNodeConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpcpb.MetricService/RpcSetNodeConfig',
            rpc__pb2.SetNodeConfigRequest.SerializeToString,
            rpc__pb2.GetNodeConfigResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



from DappleySdk.model.Utxo import Utxo


class UtxoManager:
    def __init__(self):
        pass

    @staticmethod
    def get_suitable_utxos(all_utxo, total_cost):
        if all_utxo is None:
            return None
        spendables = []
        accumulated = 0
        for utxo in all_utxo:
            if utxo is None:
                continue
            accumulated = accumulated + int(utxo.amount.encode('hex'), 16)
            temp_utxo = Utxo(utxo.amount, utxo.public_key_hash, utxo.txid, utxo.tx_index)
            spendables.append(temp_utxo)

            if accumulated - total_cost >= 0:
                break
        if accumulated - total_cost < 0:
            return None
        return spendables



import binascii
import hashlib

from DappleySdk.lib.protobuf import transaction_pb2
from DappleySdk.model.TxInput import TxInput
from DappleySdk.model.TxOutput import TxOutput
from DappleySdk.util.Constant import TxTypeContract
from DappleySdk.util.ConvertUtil import ConvertUtil


class Transaction(object):
    def __init__(self, tx_id=None, tx_inputs=None, tx_outputs=None, tip=None, gas_limit=None, gas_price=None,
                 tx_type=TxTypeContract):
        self.id = tx_id
        self.tx_inputs = tx_inputs
        self.tx_outputs = tx_outputs
        self.tip = tip
        self.gas_limit = gas_limit
        self.gas_price = gas_price
        self.type = tx_type

    def __repr__(self):
        return 'Transaction(id={0!r}, vin={1!r}, vout={2!r},tip={3!r},gas_limit={4!r},gas_price={5!r})'.format(
            self.id, self.tx_inputs, self.tx_outputs, self.tip, self.gas_limit, self.gas_price)

    def set_id(self):
        self.id = self.hash()

    def to_proto(self):
        ts = transaction_pb2.Transaction()
        ts.id = self.id
        ts.tip = ConvertUtil.int_to_bytes(self.tip)
        ts.gas_price = ConvertUtil.int_to_bytes(self.gas_price)
        ts.gas_limit = ConvertUtil.int_to_bytes(self.gas_limit)
        for tx_input in self.tx_inputs:
            ts.vin.append(tx_input.to_proto())
        for tx_output in self.tx_outputs:
            ts.vout.append(tx_output.to_proto())
        ts.type=self.type
        return ts

    def trimed_copy(self):
        inputs = []
        outputs = []

        for vin in self.tx_inputs:
            inputs.append(TxInput(vin.tx_id, vin.vout, None, None))

        for vout in self.tx_outputs:
            outputs.append(TxOutput(vout.value, vout.public_key_hash, vout.contract))

        return Transaction(self.id, inputs, outputs, self.tip, self.gas_limit, self.gas_price)

    def hash(self):
        need_hash_data = ''
        for tx_input in self.tx_inputs:
            tx_bs_bytes = ''
            if tx_input.tx_id:
                tx_bs_bytes += tx_input.tx_id
            if tx_input.vout >= 0:
                tem_data = bytes(tx_input.vout).zfill(8)
                tx_bs_bytes += binascii.a2b_hex(tem_data)
            if tx_input.pub_key:
                tx_bs_bytes += tx_input.pub_key
            if tx_input.signature:
                tx_bs_bytes += tx_input.signature
            need_hash_data = need_hash_data + tx_bs_bytes
        for tx_output in self.tx_outputs:
            tx_output_bytes = ''
            tx_output_bytes += (tx_output.value + tx_output.public_key_hash + bytes(tx_output.contract))
            need_hash_data = need_hash_data + tx_output_bytes

        need_hash_data += (ConvertUtil.int_to_bytes(self.tip) + ConvertUtil.int_to_bytes(self.gas_limit) + ConvertUtil.int_to_bytes(self.gas_price))
        need_hash_data += binascii.a2b_hex(bytes(self.type).zfill(8))
        m = hashlib.sha256()
        m.update(need_hash_data)
        _hash = m.digest()
        return _hash



from DappleySdk.lib.protobuf import transactionBase_pb2


class TxInput(object):
    def __init__(self, tx_id, vout, signature, pub_key):
        self.tx_id = tx_id
        self.vout = vout
        self.signature = signature
        self.pub_key = pub_key

    def to_proto(self):
        tx_input = transactionBase_pb2.TXInput()
        tx_input.txid = self.tx_id
        tx_input.vout = self.vout
        tx_input.signature = self.signature
        tx_input.public_key = self.pub_key
        return tx_input



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: block.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import transaction_pb2 as transaction__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='block.proto',
  package='blockpb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x0b\x62lock.proto\x12\x07\x62lockpb\x1a\x11transaction.proto\"t\n\x05\x42lock\x12$\n\x06header\x18\x01 \x01(\x0b\x32\x14.blockpb.BlockHeader\x12\x30\n\x0ctransactions\x18\x02 \x03(\x0b\x32\x1a.transactionpb.Transaction\x12\x13\n\x0bparent_hash\x18\x03 \x01(\x0c\"\x89\x01\n\x0b\x42lockHeader\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12\x15\n\rprevious_hash\x18\x02 \x01(\x0c\x12\r\n\x05nonce\x18\x03 \x01(\x03\x12\x11\n\ttimestamp\x18\x04 \x01(\x03\x12\x11\n\tsignature\x18\x05 \x01(\x0c\x12\x0e\n\x06height\x18\x06 \x01(\x04\x12\x10\n\x08producer\x18\x07 \x01(\tb\x06proto3'
  ,
  dependencies=[transaction__pb2.DESCRIPTOR,])




_BLOCK = _descriptor.Descriptor(
  name='Block',
  full_name='blockpb.Block',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='blockpb.Block.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transactions', full_name='blockpb.Block.transactions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parent_hash', full_name='blockpb.Block.parent_hash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=159,
)


_BLOCKHEADER = _descriptor.Descriptor(
  name='BlockHeader',
  full_name='blockpb.BlockHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='blockpb.BlockHeader.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='previous_hash', full_name='blockpb.BlockHeader.previous_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='blockpb.BlockHeader.nonce', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='blockpb.BlockHeader.timestamp', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='blockpb.BlockHeader.signature', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='blockpb.BlockHeader.height', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='producer', full_name='blockpb.BlockHeader.producer', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=299,
)

_BLOCK.fields_by_name['header'].message_type = _BLOCKHEADER
_BLOCK.fields_by_name['transactions'].message_type = transaction__pb2._TRANSACTION
DESCRIPTOR.message_types_by_name['Block'] = _BLOCK
DESCRIPTOR.message_types_by_name['BlockHeader'] = _BLOCKHEADER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Block = _reflection.GeneratedProtocolMessageType('Block', (_message.Message,), {
  'DESCRIPTOR' : _BLOCK,
  '__module__' : 'block_pb2'
  # @@protoc_insertion_point(class_scope:blockpb.Block)
  })
_sym_db.RegisterMessage(Block)

BlockHeader = _reflection.GeneratedProtocolMessageType('BlockHeader', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKHEADER,
  '__module__' : 'block_pb2'
  # @@protoc_insertion_point(class_scope:blockpb.BlockHeader)
  })
_sym_db.RegisterMessage(BlockHeader)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transactionBase.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='transactionBase.proto',
  package='transactionbasepb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x15transactionBase.proto\x12\x11transactionbasepb\"L\n\x07TXInput\x12\x0c\n\x04txid\x18\x01 \x01(\x0c\x12\x0c\n\x04vout\x18\x02 \x01(\x05\x12\x11\n\tsignature\x18\x03 \x01(\x0c\x12\x12\n\npublic_key\x18\x04 \x01(\x0c\"D\n\x08TXOutput\x12\r\n\x05value\x18\x01 \x01(\x0c\x12\x17\n\x0fpublic_key_hash\x18\x02 \x01(\x0c\x12\x10\n\x08\x63ontract\x18\x03 \x01(\tb\x06proto3'
)




_TXINPUT = _descriptor.Descriptor(
  name='TXInput',
  full_name='transactionbasepb.TXInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='txid', full_name='transactionbasepb.TXInput.txid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vout', full_name='transactionbasepb.TXInput.vout', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='transactionbasepb.TXInput.signature', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key', full_name='transactionbasepb.TXInput.public_key', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=44,
  serialized_end=120,
)


_TXOUTPUT = _descriptor.Descriptor(
  name='TXOutput',
  full_name='transactionbasepb.TXOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='transactionbasepb.TXOutput.value', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key_hash', full_name='transactionbasepb.TXOutput.public_key_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contract', full_name='transactionbasepb.TXOutput.contract', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=122,
  serialized_end=190,
)

DESCRIPTOR.message_types_by_name['TXInput'] = _TXINPUT
DESCRIPTOR.message_types_by_name['TXOutput'] = _TXOUTPUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TXInput = _reflection.GeneratedProtocolMessageType('TXInput', (_message.Message,), {
  'DESCRIPTOR' : _TXINPUT,
  '__module__' : 'transactionBase_pb2'
  # @@protoc_insertion_point(class_scope:transactionbasepb.TXInput)
  })
_sym_db.RegisterMessage(TXInput)

TXOutput = _reflection.GeneratedProtocolMessageType('TXOutput', (_message.Message,), {
  'DESCRIPTOR' : _TXOUTPUT,
  '__module__' : 'transactionBase_pb2'
  # @@protoc_insertion_point(class_scope:transactionbasepb.TXOutput)
  })
_sym_db.RegisterMessage(TXOutput)


# @@protoc_insertion_point(module_scope)



import binascii

from secp256k1 import PrivateKey

from DappleySdk.model.Transaction import Transaction
from DappleySdk.model.TxInput import TxInput
from DappleySdk.model.TxOutput import TxOutput
from DappleySdk.util.ConvertUtil import ConvertUtil
from DappleySdk.util.HashUtil import HashUtil


class TransactionManager:
    def __init__(self):
        pass

    def new_transaction(self, utxos, to_address, amount, private_key, tip, gas_limit, gas_price, contract):

        sign_key = PrivateKey(bytes(bytearray.fromhex(private_key)), raw=True)
        return self.inner_new_transaction(utxos, to_address, amount, sign_key, tip, gas_limit, gas_price,
                                          contract)

    def inner_new_transaction(self, utxos, to_address, amount, sign_key, tip, gas_limit, gas_price, contract):
        transaction = Transaction(None, [], [], tip, gas_limit, gas_price)
        # add vin list and return the total amount of vin values
        total_amount = self.build_vin(transaction, utxos, sign_key)

        change = self.calculate_change(total_amount, amount, tip, gas_limit, gas_price)
        if change < 0:
            return None

        # add vout list. If there is change is this transaction, vout list wound have two elements, or have just one to
        # coin receiver.
        self.build_vout(transaction, to_address, amount, change, sign_key, contract)

        # generate Id
        transaction.set_id()

        self.sign(transaction, sign_key, utxos)
        return transaction

    def build_vin(self, transaction, utxos, singn_key):
        pub_key = singn_key.pubkey.serialize(False)
        total_amount = 0
        for utxo in utxos:
            if utxo is None:
                continue

            tx_input = TxInput(utxo.tx_id, utxo.vout_index, None, pub_key[1:])
            # add from publicKey value
            total_amount = total_amount + int(utxo.amount.encode('hex'), 16)
            transaction.tx_inputs.append(tx_input)
        return total_amount

    def calculate_change(self, total_amount, amount, tip, gas_limit, gas_price):
        change = total_amount - amount
        if change < 0:
            return None
        change = change - tip
        if change < 0:
            return None
        if gas_limit is not None and gas_price is not None:
            change = change - gas_limit * gas_price
            if change < 0:
                return None
        return change

    def build_vout(self, transaction, to_address, amount, change, sign_key, contract):
        if change is not None and change < 0:
            return
        tx_output = TxOutput(b'', b'', '')
        if contract is not None:
            if to_address is None:
                # here need to generate contract address,if contract is None
                pass

            tx_output.contract = contract

        tx_output.value = ConvertUtil.int_to_bytes(amount)
        tx_output.public_key_hash = HashUtil.get_public_key_hash_with_str(to_address)
        transaction.tx_outputs.append(tx_output)

        if change is not None and change > 0:
            tx_output = TxOutput(b'', b'', '')
            # set from address's pubKeyHash
            tx_output.public_key_hash = HashUtil.get_public_key_hash_with_bytes(
                sign_key.pubkey.serialize(False))
            tx_output.value = ConvertUtil.int_to_bytes(change)
            transaction.tx_outputs.append(tx_output)

    def sign(self, transaction, sign_key, utxos):
        tx_inputs = transaction.tx_inputs
        if tx_inputs is None or len(tx_inputs) == 0:
            return
        utxo_map = self.get_prev_utxos(utxos)
        transaction_copy = transaction.trimed_copy()
        self.build_sign_value(transaction, utxo_map, transaction_copy, sign_key)

    def get_prev_utxos(self, utxos):
        utxo_map = {}
        for utxo in utxos:
            utxo_map[binascii.hexlify(utxo.tx_id) + "-" + str(utxo.vout_index)] = utxo
        return utxo_map

    def build_sign_value(self, transaction, utxo_map, transaction_copy, sign_key):
        tx_copy_inputs = transaction_copy.tx_inputs
        for i in range(0, len(tx_copy_inputs)):
            transaction.tx_inputs[i].signature = ''
            tx_input = tx_copy_inputs[i]
            old_pub_key = tx_input.pub_key
            utxo = utxo_map[binascii.hexlify(tx_input.tx_id) + "-" + str(tx_input.vout)]
            tx_input.pub_key = utxo.public_key_hash
            tx_copy_hash = transaction_copy.hash()
            tx_input.pub_key = old_pub_key
            signature = HashUtil.secp256k1_sign(tx_copy_hash, sign_key)
            transaction.tx_inputs[i].signature = signature



import binascii
import hashlib

import _pysha3
import base58

from DappleySdk.DError import ParamError
from DappleySdk.util.Constant import ADDRESS_CHECKSUM_LENGTH


class HashUtil:
    def __init__(self):
        pass

    @staticmethod
    def get_public_key_hash_with_bytes(public_key_bytes):
        v = _pysha3.sha3_256()
        v.update(public_key_bytes[1:])
        m = hashlib.new('ripemd160')
        m.update(v.digest())

        return b'\x5A'+m.digest()

    @staticmethod
    def get_public_key_hash_with_str(public_key_string):
        full_payload = base58.b58decode(public_key_string)
        return full_payload[:len(full_payload) - ADDRESS_CHECKSUM_LENGTH]

    @staticmethod
    def get_pub_key_hash_check_sum(pub_key_hash, length):
        m = hashlib.sha256()
        m2 = hashlib.sha256()
        m.update(pub_key_hash)
        first_sha = m.digest()
        m2.update(first_sha)
        second_sha = m2.digest()
        return second_sha[:length]

    @staticmethod
    def secp256k1_sign(data, sign_key):
        if len(data) != 32:
            raise ParamError(ValueError, "Expected 32 byte input to ECDSA signature, not {}".format(len(data)))
        sig_check = sign_key.ecdsa_sign_recoverable(msg=data, raw=True)
        sig_ser = sign_key.ecdsa_recoverable_serialize(sig_check)
        hex_str = "{}{}".format(binascii.hexlify(sig_ser[0]), binascii.hexlify(bytearray([sig_ser[1]])))
        return binascii.unhexlify(hex_str)



from DError import ParamError
from chain.TransactionManager import TransactionManager
from chain.UtxoManager import UtxoManager
from rpcnet.RPCClient import RPCClient


class Dappley(object):

    def __init__(self, host, port):
        self._host = host
        self._port = port

    @staticmethod
    def get_total_utxo_cost(amount, tip, gas_limit, gas_price):
        if amount is None:
            return 0
        total = amount
        total = total + tip

        if (gas_limit is not None) and (gas_price is not None):
            total = total + gas_price * gas_limit

        return total

    def send_transaction(self, from_address, to_address, amount, private_key, tip, gas_limit, gas_price, contract):
        if (from_address is None) or (to_address is None):
            raise ParamError(ValueError, "fromAddress or toAddress is empty!")

        all_utxo = RPCClient(self._host, self._port).get_utxos(from_address)
        if all_utxo is None:
            raise ParamError(ValueError, "FromAddress has no balance!")

        total_cost = self.get_total_utxo_cost(amount, tip, gas_limit, gas_price)
        utxos = UtxoManager.get_suitable_utxos(all_utxo, total_cost)

        if utxos is None:
            raise ParamError(ValueError, "Balance of fromAddress is not enough!")

        transaction = TransactionManager().new_transaction(utxos, to_address, amount, private_key, tip, gas_limit,
                                                           gas_price, contract)
        request_data = transaction.to_proto()
        send_tx_result = RPCClient(self._host, self._port).send_transaction(request_data)
        return send_tx_result


if __name__ == '__main__':
    pass



class ConvertUtil:
    def __init__(self):
        pass

    @staticmethod
    def int_to_bytes(value):
        data = "{0:b}".format(value)
        return ConvertUtil.bitstring_to_bytes(data)

    @staticmethod
    def bitstring_to_bytes(s):
        v = int(s, 2)
        b = bytearray()
        while v:
            b.append(v & 0xff)
            v >>= 8
        return bytes(b[::-1])


import binascii

from DappleySdk.lib.protobuf import transaction_pb2, rpc_pb2
from DappleySdk.model.Transaction import Transaction
from DappleySdk.rpcnet.RPCClient import RPCClient


def test_get_version():
    print ("==============test get version start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    version_info = rpc.get_version("1.0.0")
    print (version_info)
    print ("==============test get version end===========\n")


def test_get_balance():
    print ("==============test get balance start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    balance = rpc.get_balance("dastXXWLe5pxbRYFhcyUq8T3wb5srWkHKa")
    print ("balance:{}".format(balance))
    print ("==============test get balance end===========\n")


def test_get_block_chain_info():
    print ("==============test get block chain info start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    block_chain_info = rpc.get_block_chain_info()
    print ("block_chain_info:{}".format(block_chain_info))
    print ("==============test get block chain info end===========\n")


def test_get_utxos():
    print ("==============test get utxos start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    utxos = rpc.get_utxos("dastXXWLe5pxbRYFhcyUq8T3wb5srWkHKa")
    print ("utxos len:{}".format(len(utxos)))
    print ("==============test get utxos end===========\n")


def test_get_blocks():
    print ("==============test get blocks start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    blocks = rpc.get_blocks("821fe48f468eca9800a4d15ff2091b20cc76732b18e25eef349013509e1d3868", 2)
    print ("blocks :{}".format(blocks))
    print ("==============test get blocks end===========\n")


def test_get_block_by_hash():
    print ("==============test get block by hash start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    block = rpc.get_block_by_hash(binascii.a2b_hex("821fe48f468eca9800a4d15ff2091b20cc76732b18e25eef349013509e1d3868"))
    print ("block :{}".format(block))
    print ("==============test get block by hash end===========\n")


def test_get_block_by_height():
    print("==============test get block by height start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    block = rpc.get_block_by_height(1)
    print ("block :{}".format(block))
    print ("==============test get block by height end===========\n")


def test_get_gas_price():
    print("==============test get gas_price start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    gas_price = rpc.get_gas_price()
    print ("gas_price :{}".format(binascii.hexlify(gas_price)))
    print ("==============test get gas_price end===========\n")


def test_send_transaction():
    print("==============test send_transaction start=========")
    rpc = RPCClient("127.0.0.1", "50050")
    transaction = Transaction()
    transaction.gas_limit = 0
    data = transaction.to_proto()
    send_transaction = rpc.send_transaction(data)
    print ("send_transaction :{}".format(send_transaction))
    print ("==============test send_transaction end===========\n")


def test_all_run():
    test_get_version()
    test_get_balance()
    test_get_block_chain_info()
    test_get_utxos()
    test_get_blocks()
    test_get_block_by_hash()
    test_get_block_by_height()
    test_get_gas_price()
    test_send_transaction()


if __name__ == '__main__':
    print ("start to test rpc client and each api")
    test_all_run()
    print ("test rpc client end")



import binascii
import grpc
from DappleySdk.lib.protobuf import rpc_pb2_grpc, rpc_pb2


class RPCClient:
    def __init__(self, host, port):
        self._host = host
        self._port = port

    def shut_down_chanel(self, channel):
        channel.close()

    def get_channel(self):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        return channel

    def get_rpc_client(self, channel):
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        return stub

    def get_version(self, proto_version):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetVersion(rpc_pb2.GetVersionRequest(proto_version=proto_version))
        message = "{}'protocolVersion':{},'serverVersion':{}{}".format("{", response.proto_version,
                                                                       response.server_version, "}")
        channel.close()
        return message

    def get_balance(self, address):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetBalance(rpc_pb2.GetBalanceRequest(address=address))
        channel.close()
        return response.amount

    def get_block_chain_info(self):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetBlockchainInfo(rpc_pb2.GetBlockchainInfoRequest())
        channel.close()
        return {"tailBlockHash": binascii.hexlify(response.tail_block_hash), "blockHeight": response.block_height,
                "producers": response.producers}

    def get_utxos(self, address):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetUTXO(rpc_pb2.GetUTXORequest(address=address))
        channel.close()
        return response.utxos

    def get_blocks(self, start_block_hashes, max_count):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetBlocks(
            rpc_pb2.GetBlocksRequest(start_block_hashes=start_block_hashes, max_count=max_count))
        channel.close()
        return response.blocks

    def get_block_by_hash(self, byte_hash):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetBlockByHash(rpc_pb2.GetBlockByHashRequest(hash=byte_hash))
        channel.close()
        return response.block

    def get_block_by_height(self, height):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGetBlockByHeight(rpc_pb2.GetBlockByHeightRequest(height=height))
        channel.close()
        return response.block

    def send_transaction(self, transaction):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        try:
            response = stub.RpcSendTransaction(rpc_pb2.SendTransactionRequest(transaction=transaction))
            channel.close()
            return {"code": 0, "message": response.generated_contract_address}
        except grpc.RpcError as e:
            return {"code": -1, "message": e.details()}

    def estimate_gas(self, transaction):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcEstimateGas(rpc_pb2.EstimateGasRequest(transaction=transaction))
        channel.close()
        return response.gas_count

    def get_gas_price(self):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcGasPrice(rpc_pb2.GasPriceRequest())
        channel.close()
        return response.gas_price

    def contract_query(self, contract_address, key, value):
        channel = grpc.insecure_channel('{}:{}'.format(self._host, self._port))
        stub = rpc_pb2_grpc.RpcServiceStub(channel)
        response = stub.RpcContractQuery(
            rpc_pb2.ContractQueryRequest(contract_addr=contract_address, key=key, value=value))
        channel.close()
        return {"key": response.key, "value": response.value}



import binascii

import base58
from DappleySdk.util.Constant import ADDRESS_CHECKSUM_LENGTH
from DappleySdk.util.HashUtil import HashUtil


class AddressUtil:
    def __init__(self):
        pass

    @staticmethod
    def validate_address(address):
        if address is None:
            return False
        full_payload = base58.b58decode(address)

        if (full_payload is None) or (len(full_payload) < ADDRESS_CHECKSUM_LENGTH):
            return False
        actual_check_sum = full_payload[len(full_payload)-ADDRESS_CHECKSUM_LENGTH:]

        pub_key_hash = full_payload[:len(full_payload)-ADDRESS_CHECKSUM_LENGTH]
        check_sum = HashUtil.get_pub_key_hash_check_sum(pub_key_hash, ADDRESS_CHECKSUM_LENGTH)
        return actual_check_sum == check_sum



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transaction.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import transactionBase_pb2 as transactionBase__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='transaction.proto',
  package='transactionpb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x11transaction.proto\x12\rtransactionpb\x1a\x15transactionBase.proto\"@\n\x0cTransactions\x12\x30\n\x0ctransactions\x18\x01 \x03(\x0b\x32\x1a.transactionpb.Transaction\"\xae\x01\n\x0bTransaction\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\'\n\x03vin\x18\x02 \x03(\x0b\x32\x1a.transactionbasepb.TXInput\x12)\n\x04vout\x18\x03 \x03(\x0b\x32\x1b.transactionbasepb.TXOutput\x12\x0b\n\x03tip\x18\x04 \x01(\x0c\x12\x11\n\tgas_limit\x18\x05 \x01(\x0c\x12\x11\n\tgas_price\x18\x06 \x01(\x0c\x12\x0c\n\x04type\x18\x07 \x01(\x05\"\xd7\x01\n\x0fTransactionNode\x12>\n\x08\x63hildren\x18\x01 \x03(\x0b\x32,.transactionpb.TransactionNode.ChildrenEntry\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.transactionpb.Transaction\x12\x0c\n\x04size\x18\x03 \x01(\x03\x1aK\n\rChildrenEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.transactionpb.Transaction:\x02\x38\x01\"?\n\x12TransactionJournal\x12)\n\x04vout\x18\x01 \x03(\x0b\x32\x1b.transactionbasepb.TXOutputb\x06proto3'
  ,
  dependencies=[transactionBase__pb2.DESCRIPTOR,])




_TRANSACTIONS = _descriptor.Descriptor(
  name='Transactions',
  full_name='transactionpb.Transactions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='transactions', full_name='transactionpb.Transactions.transactions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=59,
  serialized_end=123,
)


_TRANSACTION = _descriptor.Descriptor(
  name='Transaction',
  full_name='transactionpb.Transaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='transactionpb.Transaction.id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vin', full_name='transactionpb.Transaction.vin', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vout', full_name='transactionpb.Transaction.vout', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tip', full_name='transactionpb.Transaction.tip', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_limit', full_name='transactionpb.Transaction.gas_limit', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gas_price', full_name='transactionpb.Transaction.gas_price', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='transactionpb.Transaction.type', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=126,
  serialized_end=300,
)


_TRANSACTIONNODE_CHILDRENENTRY = _descriptor.Descriptor(
  name='ChildrenEntry',
  full_name='transactionpb.TransactionNode.ChildrenEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='transactionpb.TransactionNode.ChildrenEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='transactionpb.TransactionNode.ChildrenEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=518,
)

_TRANSACTIONNODE = _descriptor.Descriptor(
  name='TransactionNode',
  full_name='transactionpb.TransactionNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='children', full_name='transactionpb.TransactionNode.children', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='transactionpb.TransactionNode.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='transactionpb.TransactionNode.size', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TRANSACTIONNODE_CHILDRENENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=303,
  serialized_end=518,
)


_TRANSACTIONJOURNAL = _descriptor.Descriptor(
  name='TransactionJournal',
  full_name='transactionpb.TransactionJournal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vout', full_name='transactionpb.TransactionJournal.vout', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=520,
  serialized_end=583,
)

_TRANSACTIONS.fields_by_name['transactions'].message_type = _TRANSACTION
_TRANSACTION.fields_by_name['vin'].message_type = transactionBase__pb2._TXINPUT
_TRANSACTION.fields_by_name['vout'].message_type = transactionBase__pb2._TXOUTPUT
_TRANSACTIONNODE_CHILDRENENTRY.fields_by_name['value'].message_type = _TRANSACTION
_TRANSACTIONNODE_CHILDRENENTRY.containing_type = _TRANSACTIONNODE
_TRANSACTIONNODE.fields_by_name['children'].message_type = _TRANSACTIONNODE_CHILDRENENTRY
_TRANSACTIONNODE.fields_by_name['value'].message_type = _TRANSACTION
_TRANSACTIONJOURNAL.fields_by_name['vout'].message_type = transactionBase__pb2._TXOUTPUT
DESCRIPTOR.message_types_by_name['Transactions'] = _TRANSACTIONS
DESCRIPTOR.message_types_by_name['Transaction'] = _TRANSACTION
DESCRIPTOR.message_types_by_name['TransactionNode'] = _TRANSACTIONNODE
DESCRIPTOR.message_types_by_name['TransactionJournal'] = _TRANSACTIONJOURNAL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Transactions = _reflection.GeneratedProtocolMessageType('Transactions', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTIONS,
  '__module__' : 'transaction_pb2'
  # @@protoc_insertion_point(class_scope:transactionpb.Transactions)
  })
_sym_db.RegisterMessage(Transactions)

Transaction = _reflection.GeneratedProtocolMessageType('Transaction', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTION,
  '__module__' : 'transaction_pb2'
  # @@protoc_insertion_point(class_scope:transactionpb.Transaction)
  })
_sym_db.RegisterMessage(Transaction)

TransactionNode = _reflection.GeneratedProtocolMessageType('TransactionNode', (_message.Message,), {

  'ChildrenEntry' : _reflection.GeneratedProtocolMessageType('ChildrenEntry', (_message.Message,), {
    'DESCRIPTOR' : _TRANSACTIONNODE_CHILDRENENTRY,
    '__module__' : 'transaction_pb2'
    # @@protoc_insertion_point(class_scope:transactionpb.TransactionNode.ChildrenEntry)
    })
  ,
  'DESCRIPTOR' : _TRANSACTIONNODE,
  '__module__' : 'transaction_pb2'
  # @@protoc_insertion_point(class_scope:transactionpb.TransactionNode)
  })
_sym_db.RegisterMessage(TransactionNode)
_sym_db.RegisterMessage(TransactionNode.ChildrenEntry)

TransactionJournal = _reflection.GeneratedProtocolMessageType('TransactionJournal', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTIONJOURNAL,
  '__module__' : 'transaction_pb2'
  # @@protoc_insertion_point(class_scope:transactionpb.TransactionJournal)
  })
_sym_db.RegisterMessage(TransactionJournal)


_TRANSACTIONNODE_CHILDRENENTRY._options = None
# @@protoc_insertion_point(module_scope)



class Utxo(object):
    def __init__(self, amount, public_key_hash, tx_id, vout_index):
        self.amount = amount
        self.public_key_hash = public_key_hash
        self.tx_id = tx_id
        self.vout_index = vout_index
